﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.Core;

namespace WpfAppDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void mapView_Loaded(object sender, RoutedEventArgs e)
        {
            mapView.MapUnit = GeographyUnit.Meter;
            mapView.ZoomLevelSet = new ZoomLevelSet();
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel07);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel08);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel09);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel10);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel11);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel12);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel13);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel14);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel15);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel16);
            mapView.ZoomLevelSet.CustomZoomLevels.Add(mapView.ZoomLevelSet.ZoomLevel17);

            var wmtsLayer = new WmtsLayer();
            wmtsLayer.SendingWebRequest += WmtsLayer_SendingWebRequest;

            wmtsLayer.DrawingExceptionMode = DrawingExceptionMode.DrawException;
            wmtsLayer.WmtsSeverEncodingType = WmtsSeverEncodingType.Kvp;
            wmtsLayer.ServerUris.Add(new Uri("https://api.os.uk/maps/raster/v1/wmts"));
            wmtsLayer.Open();
            wmtsLayer.ActiveLayerName = "Road_3857";
            wmtsLayer.ActiveStyleName = "default";
            wmtsLayer.TileMatrixSetName = "EPSG:3857";
            wmtsLayer.OutputFormat = "image/png";
            // You can view the capabilities. Above value are base on the capabilities
            var x = wmtsLayer.GetServerCapabilitiesXml();

            var layerOverlay = new LayerOverlay();
            layerOverlay.TileCache = new FileRasterTileCache(@"C:\\temp");
            layerOverlay.Layers.Add(wmtsLayer);
            mapView.Overlays.Clear();
            mapView.Overlays.Add("layerOverlay", layerOverlay);
            
            // Set suitable extent
            var extent4326 = new RectangleShape(-8.764179999999964, 55.3311510086, 2.9134115552, 45.52842300000003);
            ProjectionConverter proj = new ProjectionConverter(4326, 3857);
            proj.Open();
            mapView.CurrentExtent = proj.ConvertToExternalProjection(extent4326);

            mapView.Refresh();
        }

        private void WmtsLayer_SendingWebRequest(object sender, SendingWebRequestEventArgs e)
        {
            string uri = e.WebRequest.RequestUri.AbsoluteUri;
            if (uri.Contains("?"))
            {
                uri = uri + $"&key=ZsSPHhEhQUgspZCTyYmvGVqPGAtMg3DN";
            }
            else
            {
                uri = uri + $"?key=ZsSPHhEhQUgspZCTyYmvGVqPGAtMg3DN";
            }
            // set 7 offset for the zoom level.
            var zIndex = HttpUtility.ParseQueryString(uri).Get("TileMatrix");
            if (zIndex != null)
            {
                var newZindex = int.Parse(zIndex) + 7;
                uri = uri.Replace($"&TileMatrix={zIndex}&", $"&TileMatrix={newZindex}&");
            }

            System.Diagnostics.Trace.WriteLine(uri);
            e.WebRequest = HttpWebRequest.Create(uri);
        }
    }
}
